/*
 * Copyright (c) 2003-6 Viliam Holub
 */


#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>

#include "text.h"
#include "termf.h"


/* This variable informs whether output terminal is in raw mode. */
bool raw = false; 


/** Shows a message and exits.
 */
void
die( int exc, const char *s, ...)

{
	va_list ap;
	va_start( ap, s);

	term_done();
	line_done();

	vfprintf( stderr, s, ap);
	va_end( ap);
	fprintf( stderr, "\n");

	exit( exc);
}


/* Die - called when there is an incorrect parameter
 */
void
die_parm( const char *par, const char *s)
	
{
	die( RET_PAR, "%s: %s: %s", txt_progname, par, s);
}


/** Die with errno specification.
 */
void
die_errno( void)
	
{
	die( RET_IO, "%s: %s", txt_progname, strerror( errno));
}


/** Die with errno specification and additional text.
 */
void
die_errno_spec( const char *spec)

{
	die( RET_IO, "%s: %s: %s", txt_progname, spec, strerror( errno));
}


/** Shows a message.
 */
void
info( const char *s, ...)

{
	va_list ap;
	va_start( ap, s);

	vfprintf( stderr, s, ap);
	va_end( ap);
	fprintf( stderr, raw ? "\r\n" : "\n");
}


/** Shows errno specification and additional text.
 */
void
info_errno_spec( const char *spec)

{
	info( "%s: %s: %s", txt_progname, spec, strerror( errno));
}
