/*
 * Copyright (c) 2003-2006 Viliam Holub
 */

#include <cross.h>
#include <unistd.h>
#include <fcntl.h>

#include "termf.h"
#include "cross.h"
#include "parse.h"
#include "fault.h"
#include "safecall.h"
#include "text.h"


bool is_stdin_tty = false;
bool is_stdout_tty = false;
bool is_vt_tty = false;


/* lk have to use 2 inputs and 1 output. The first input is the keyboard
 * (standard input), the second one is from remote system. Output is to
 * standard output.
 */
struct termios tio, tio_vt;
struct termios tio_stdin;
struct termios tio_stdin_raw;
struct termios tio_stdout;
struct termios tio_stdout_raw;

/* Serial handler. */
int line;



/** Configures terminal.
 *
 * Terminal is configured to receive inputs and outputs as raw data without
 * interpretation.
 */
void
term_init( void)

{
	bool iit, iot;

	iit = isatty( 0);
	iot = isatty( 1);
		
	if (iit)
	{
		s_tcgetattr( 0, &tio_stdin);
		is_stdin_tty = true;
	}
		
	if (iot)
	{
		s_tcgetattr( 1, &tio_stdout);
		is_stdout_tty = true;
	}
	
	if (iit)
	{	
		tio_stdin_raw = tio_stdin;
		
		tio_stdin_raw.c_lflag &= ~(ISIG|ICANON|ECHO|IEXTEN);
		tio_stdin_raw.c_iflag |= IGNPAR;
		tio_stdin_raw.c_iflag &= ~(IGNBRK|INLCR|IGNCR|ICRNL|IXON|IXOFF|INPCK|ISTRIP);
		
		tio_stdin_raw.c_cc[ VMIN] = 1;
		tio_stdin_raw.c_cc[ VTIME] = 1;
		
		s_tcsetattr( 0, TCSADRAIN, &tio_stdin_raw);
	}
	
	if (iot)
	{	
		tio_stdout_raw = tio_stdout;
		
		tio_stdout_raw.c_lflag &= ~(ISIG|ICANON|ECHO|IEXTEN);
		tio_stdout_raw.c_oflag &= ~OPOST;
		
		tio_stdout_raw.c_cc[ VMIN] = 1;
		tio_stdout_raw.c_cc[ VTIME] = 1;
		
		s_tcsetattr( 1, TCSADRAIN, &tio_stdout_raw);
		
		raw = true;
	}
}


/** Restores old termio atributes.
 */
void
term_done( void)

{
	term_restore();
}


/** Sets the terminal back to raw mode.
 */
void
term_set( void)

{
	if (is_stdin_tty)
		s_tcsetattr( 0, TCSADRAIN, &tio_stdin_raw);
	if (is_stdout_tty)
	{
		s_tcsetattr( 1, TCSADRAIN, &tio_stdout_raw);
		raw = true;
	}
}


/** Restores the terminal to normal mode.
 */
void
term_restore( void)

{
	if (is_stdin_tty)
		tcsetattr( 0, TCSANOW, &tio_stdin);
	if (is_stdout_tty)
	{
		tcsetattr( 1, TCSANOW, &tio_stdout);
		raw = false;
	}
}


/** Inits device_name line.
 */
void
line_init( void)

{
	/*
	 * Opening the line
	 */
	line = open( device_name, O_RDWR);
	if (line == -1)
		die_errno_spec( device_name);
	is_vt_tty = isatty( line);

	if (!is_vt_tty)
		return;
	
	/*
	 * Setting up the line
	 */
	s_tcgetattr( line, &tio);
	
	tio_vt = tio;
	
	tio_vt.c_lflag &= ~(ISIG|ICANON|ECHO|IEXTEN);
	tio_vt.c_iflag |= IGNBRK|IGNPAR;
	
	/* carrier control */
	if (carrier_control != undef)
	{
		tio_vt.c_cflag &= ~CLOCAL;
		if (!carrier_control)
			tio_vt.c_cflag |= CLOCAL;
	}
	
	/* software flow control */
	if (ixon != undef)
	{
		tio_vt.c_iflag &= ~IXON;
		if (ixon)
			tio_vt.c_iflag |= IXON;
	}
	if (ixoff != undef)
	{
		tio_vt.c_iflag &= ~IXOFF;
		if (ixoff)
			tio_vt.c_iflag |= IXOFF;
	}
	if (!ixoff && !ixon)
		tio_vt.c_iflag &= ~IXANY;
	
	/* hardware flow control */
	if ((flow_control_input != undef) || (flow_control_output != undef))
	{
		if (flow_control_input == flow_control_output)
		{
#ifdef HAVE_CRTSCTS
		tio_vt.c_cflag &= ~CRTSCTS;
		if (flow_control_input)
			tio_vt.c_cflag |= CRTSCTS;
#else
# ifdef HAVE_CRTS_IFLOW && HAVE_CCTS_OFLOW
		tio_vt.c_cflag &= ~(CRTS_IFLOW || CCTS_OFLOW);
		if (flow_control_input)
			tio_vt.c_cflag |= CRTS_IFLOW || CCTS_OFLOW;
# else
		/* huh */
		die_parm( "fc", txt_fcuns);
# endif
#endif
		}
		else
		{
			if (flow_control_input != undef)
			{
#ifdef CRTS_IFLOW
				tio_vt.c_cflag &= ~CRTS_IFLOW;
				if (flow_control_input)
					tio_vt.c_cflag |= CRTS_IFLOW;
#else
				die_parm( "fci", txt_fciuns);
#endif
			}
			
			if (flow_control_output != undef)
			{
#ifdef CCTS_OFLOW
				tio_vt.c_cflag &= ~CCTS_IFLOW;
				if (flow_control_output)
					tio_vt.c_cflag |= CCTS_OFLOW;
#else
				die_parm( "fco", txt_fcouns);
#endif
			}
		}
	}
	
	/* stop bits */
	if (double_stop != undef)
	{
		tio_vt.c_cflag &= ~(CSTOPB);
		if (double_stop)
			tio_vt.c_cflag |= CSTOPB;
	}
	
	/* parity */
	if (parity != -1)
	{
		tio_vt.c_cflag &= ~(PARENB | PARODD);
		tio_vt.c_cflag |= parity;
	}
	
	/* char size */
	if (character_size != -1)
	{
		unsigned i = 0;
		tio_vt.c_cflag &= ~CSIZE;
		
		switch (character_size)
		{
			case '5':
				i = CS5;
				break;
			case '6':
				i = CS6;
				break;
			case '7':
				i = CS7;
				break;
			case '8':
				i = CS8;
				break;
		}
		
		tio_vt.c_cflag |= i;
	}

	/* baud rate */
	if (baud_rate_set)
	{
		s_cfsetospeed( &tio_vt, baud_rate);
		s_cfsetispeed( &tio_vt, baud_rate);
	}
	

	tio_vt.c_iflag &= ~(IGNBRK|INLCR|IGNCR|ICRNL|INPCK|ISTRIP);
	
	tio_vt.c_cflag |= CREAD;
	tio_vt.c_cflag &= ~HUPCL;
	
	tio_vt.c_oflag &= ~OPOST;
	
	tio_vt.c_cc[ VMIN] = 1;
	tio_vt.c_cc[ VTIME] = 0;

	s_tcsetattr( line, TCSANOW, &tio_vt);
}


/** Sets old attributes and closes file.
 */
void
line_done( void)

{
	if (is_vt_tty)
	{
		tcsetattr( line, TCSANOW, &tio);
		close( line);
	}
}


/** Hangs up the line.
 *
 * Hang-up deactivates DTR for a while.
 */
void
line_hangup( int recover)

{
	info( "[Sending hangup signal...]");
	if (is_vt_tty)
	{
		struct termios t = tio_vt;

		/* Set baudrate to 0 - hangup */
		s_cfsetospeed( &t, B0);
		s_cfsetispeed( &t, B0);
		s_tcsetattr( line, TCSADRAIN, &t);

		/* Wait a while. */
		sleep( 1);

		/* Recover line. */
		if (recover == DO_RECOVER)
			s_tcsetattr( line, TCSANOW, &tio_vt);
	}
}


/** Sends serial break.
 *
 * Serial break is a transmission of a continuous stream of zero-valued bits. Used as
 * s system-request kay.
 */
void
line_break( void)

{
	if (is_vt_tty)
		s_tcsendbreak( line, 0);
}
